# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["AccountListParams"]


class AccountListParams(TypedDict, total=False):
    cursor: str
    """The pagination cursor value."""

    customer_slug: str
    """Customer slug"""

    deployment_status: Literal["deployed", "migrating", "pre-deployment", "retired", "sales", "test"]
    """
    - `deployed` - Deployed
    - `pre-deployment` - Pre Deployment
    - `migrating` - Migrating
    - `sales` - Sales
    - `test` - Test
    - `retired` - Retired
    """

    environment: Optional[Literal["Demo", "dev", "lower_envs", "production", "qa", "staging", "uat"]]
    """
    - `production` - Production
    - `staging` - Staging
    - `qa` - QA
    - `dev` - Dev
    - `uat` - UAT
    - `lower_envs` - Lower Environments
    - `Demo` - Demo
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import experiment_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorURLPage, AsyncCursorURLPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.experiment import Experiment

__all__ = ["ExperimentsResource", "AsyncExperimentsResource"]


class ExperimentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ExperimentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#accessing-raw-response-data-eg-headers
        """
        return ExperimentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExperimentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#with_streaming_response
        """
        return ExperimentsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        experiment_id: int,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experiment:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            f"/api/external/v1/experiments/{experiment_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experiment,
        )

    def list(
        self,
        *,
        account_slug: str | Omit = omit,
        complete_state: Optional[Literal["accepted", "hidden", "invalid", "null", "rejected"]] | Omit = omit,
        cursor: str | Omit = omit,
        customer_slug: str | Omit = omit,
        ended_at_after: Union[str, datetime] | Omit = omit,
        ended_at_before: Union[str, datetime] | Omit = omit,
        has_ended: bool | Omit = omit,
        has_started: bool | Omit = omit,
        section_id: int | Omit = omit,
        started_at_after: Union[str, datetime] | Omit = omit,
        started_at_before: Union[str, datetime] | Omit = omit,
        status: Literal[
            "ended",
            "failed-salt-shake",
            "paused",
            "published",
            "ready",
            "rolledout",
            "waiting-for-end",
            "waiting-for-manual-salt",
            "waiting-for-rollout",
            "waiting-for-salt",
        ]
        | Omit = omit,
        tags_all: SequenceNotStr[str] | Omit = omit,
        tags_any: SequenceNotStr[str] | Omit = omit,
        test_type: Literal["cro", "full_funnel", "linear_full_funnel", "seo"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[Experiment]:
        """
        Args:
          account_slug: Account slug

          complete_state: - `accepted` - Positive
              - `rejected` - Negative
              - `null` - Inconclusive
              - `invalid` - Invalid
              - `hidden` - Hidden

          cursor: The pagination cursor value.

          customer_slug: Customer slug

          ended_at_after: Ended at

          ended_at_before: Ended at

          has_ended: Has Ended

          has_started: Has Started

          section_id: Section ID

          started_at_after: Started At

          started_at_before: Started At

          status: - `waiting-for-salt` - Waiting for salt
              - `waiting-for-manual-salt` - Waiting for manual salt
              - `failed-salt-shake` - Failed salt shake
              - `ready` - Ready
              - `published` - Published
              - `paused` - Paused
              - `waiting-for-rollout` - Waiting for rollout
              - `rolledout` - Rolled out to 100%
              - `waiting-for-end` - Waiting for end
              - `ended` - Ended

          tags_all: Filter for experiments who match ALL tag names

          tags_any: Filter for experiments who match ANY tag names

          test_type: - `seo` - SEO
              - `cro` - CRO
              - `full_funnel` - Full Funnel
              - `linear_full_funnel` - Linear Full Funnel

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/external/v1/experiments/",
            page=SyncCursorURLPage[Experiment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_slug": account_slug,
                        "complete_state": complete_state,
                        "cursor": cursor,
                        "customer_slug": customer_slug,
                        "ended_at_after": ended_at_after,
                        "ended_at_before": ended_at_before,
                        "has_ended": has_ended,
                        "has_started": has_started,
                        "section_id": section_id,
                        "started_at_after": started_at_after,
                        "started_at_before": started_at_before,
                        "status": status,
                        "tags_all": tags_all,
                        "tags_any": tags_any,
                        "test_type": test_type,
                    },
                    experiment_list_params.ExperimentListParams,
                ),
            ),
            model=Experiment,
        )


class AsyncExperimentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncExperimentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#accessing-raw-response-data-eg-headers
        """
        return AsyncExperimentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExperimentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#with_streaming_response
        """
        return AsyncExperimentsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        experiment_id: int,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Experiment:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            f"/api/external/v1/experiments/{experiment_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Experiment,
        )

    def list(
        self,
        *,
        account_slug: str | Omit = omit,
        complete_state: Optional[Literal["accepted", "hidden", "invalid", "null", "rejected"]] | Omit = omit,
        cursor: str | Omit = omit,
        customer_slug: str | Omit = omit,
        ended_at_after: Union[str, datetime] | Omit = omit,
        ended_at_before: Union[str, datetime] | Omit = omit,
        has_ended: bool | Omit = omit,
        has_started: bool | Omit = omit,
        section_id: int | Omit = omit,
        started_at_after: Union[str, datetime] | Omit = omit,
        started_at_before: Union[str, datetime] | Omit = omit,
        status: Literal[
            "ended",
            "failed-salt-shake",
            "paused",
            "published",
            "ready",
            "rolledout",
            "waiting-for-end",
            "waiting-for-manual-salt",
            "waiting-for-rollout",
            "waiting-for-salt",
        ]
        | Omit = omit,
        tags_all: SequenceNotStr[str] | Omit = omit,
        tags_any: SequenceNotStr[str] | Omit = omit,
        test_type: Literal["cro", "full_funnel", "linear_full_funnel", "seo"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Experiment, AsyncCursorURLPage[Experiment]]:
        """
        Args:
          account_slug: Account slug

          complete_state: - `accepted` - Positive
              - `rejected` - Negative
              - `null` - Inconclusive
              - `invalid` - Invalid
              - `hidden` - Hidden

          cursor: The pagination cursor value.

          customer_slug: Customer slug

          ended_at_after: Ended at

          ended_at_before: Ended at

          has_ended: Has Ended

          has_started: Has Started

          section_id: Section ID

          started_at_after: Started At

          started_at_before: Started At

          status: - `waiting-for-salt` - Waiting for salt
              - `waiting-for-manual-salt` - Waiting for manual salt
              - `failed-salt-shake` - Failed salt shake
              - `ready` - Ready
              - `published` - Published
              - `paused` - Paused
              - `waiting-for-rollout` - Waiting for rollout
              - `rolledout` - Rolled out to 100%
              - `waiting-for-end` - Waiting for end
              - `ended` - Ended

          tags_all: Filter for experiments who match ALL tag names

          tags_any: Filter for experiments who match ANY tag names

          test_type: - `seo` - SEO
              - `cro` - CRO
              - `full_funnel` - Full Funnel
              - `linear_full_funnel` - Linear Full Funnel

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/external/v1/experiments/",
            page=AsyncCursorURLPage[Experiment],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_slug": account_slug,
                        "complete_state": complete_state,
                        "cursor": cursor,
                        "customer_slug": customer_slug,
                        "ended_at_after": ended_at_after,
                        "ended_at_before": ended_at_before,
                        "has_ended": has_ended,
                        "has_started": has_started,
                        "section_id": section_id,
                        "started_at_after": started_at_after,
                        "started_at_before": started_at_before,
                        "status": status,
                        "tags_all": tags_all,
                        "tags_any": tags_any,
                        "test_type": test_type,
                    },
                    experiment_list_params.ExperimentListParams,
                ),
            ),
            model=Experiment,
        )


class ExperimentsResourceWithRawResponse:
    def __init__(self, experiments: ExperimentsResource) -> None:
        self._experiments = experiments

        self.retrieve = to_raw_response_wrapper(
            experiments.retrieve,
        )
        self.list = to_raw_response_wrapper(
            experiments.list,
        )


class AsyncExperimentsResourceWithRawResponse:
    def __init__(self, experiments: AsyncExperimentsResource) -> None:
        self._experiments = experiments

        self.retrieve = async_to_raw_response_wrapper(
            experiments.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            experiments.list,
        )


class ExperimentsResourceWithStreamingResponse:
    def __init__(self, experiments: ExperimentsResource) -> None:
        self._experiments = experiments

        self.retrieve = to_streamed_response_wrapper(
            experiments.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            experiments.list,
        )


class AsyncExperimentsResourceWithStreamingResponse:
    def __init__(self, experiments: AsyncExperimentsResource) -> None:
        self._experiments = experiments

        self.retrieve = async_to_streamed_response_wrapper(
            experiments.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            experiments.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Account"]


class Account(BaseModel):
    id: int

    customer_slug: str

    deployment_status: Literal["deployed", "pre-deployment", "migrating", "sales", "test", "retired"]
    """
    - `deployed` - Deployed
    - `pre-deployment` - Pre Deployment
    - `migrating` - Migrating
    - `sales` - Sales
    - `test` - Test
    - `retired` - Retired
    """

    domain_name: str
    """Domain name of the site (excluding .live.spodn.com)"""

    name: str

    slug: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import account_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorURLPage, AsyncCursorURLPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.account import Account

__all__ = ["AccountsResource", "AsyncAccountsResource"]


class AccountsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#accessing-raw-response-data-eg-headers
        """
        return AccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#with_streaming_response
        """
        return AccountsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        account_slug: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Account:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_slug:
            raise ValueError(f"Expected a non-empty value for `account_slug` but received {account_slug!r}")
        return self._get(
            f"/api/external/v1/accounts/{account_slug}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Account,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        customer_slug: str | Omit = omit,
        deployment_status: Literal["deployed", "migrating", "pre-deployment", "retired", "sales", "test"] | Omit = omit,
        environment: Optional[Literal["Demo", "dev", "lower_envs", "production", "qa", "staging", "uat"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[Account]:
        """
        Args:
          cursor: The pagination cursor value.

          customer_slug: Customer slug

          deployment_status: - `deployed` - Deployed
              - `pre-deployment` - Pre Deployment
              - `migrating` - Migrating
              - `sales` - Sales
              - `test` - Test
              - `retired` - Retired

          environment: - `production` - Production
              - `staging` - Staging
              - `qa` - QA
              - `dev` - Dev
              - `uat` - UAT
              - `lower_envs` - Lower Environments
              - `Demo` - Demo

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/external/v1/accounts/",
            page=SyncCursorURLPage[Account],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "customer_slug": customer_slug,
                        "deployment_status": deployment_status,
                        "environment": environment,
                    },
                    account_list_params.AccountListParams,
                ),
            ),
            model=Account,
        )


class AsyncAccountsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#with_streaming_response
        """
        return AsyncAccountsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        account_slug: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Account:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_slug:
            raise ValueError(f"Expected a non-empty value for `account_slug` but received {account_slug!r}")
        return await self._get(
            f"/api/external/v1/accounts/{account_slug}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Account,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        customer_slug: str | Omit = omit,
        deployment_status: Literal["deployed", "migrating", "pre-deployment", "retired", "sales", "test"] | Omit = omit,
        environment: Optional[Literal["Demo", "dev", "lower_envs", "production", "qa", "staging", "uat"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Account, AsyncCursorURLPage[Account]]:
        """
        Args:
          cursor: The pagination cursor value.

          customer_slug: Customer slug

          deployment_status: - `deployed` - Deployed
              - `pre-deployment` - Pre Deployment
              - `migrating` - Migrating
              - `sales` - Sales
              - `test` - Test
              - `retired` - Retired

          environment: - `production` - Production
              - `staging` - Staging
              - `qa` - QA
              - `dev` - Dev
              - `uat` - UAT
              - `lower_envs` - Lower Environments
              - `Demo` - Demo

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/external/v1/accounts/",
            page=AsyncCursorURLPage[Account],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "customer_slug": customer_slug,
                        "deployment_status": deployment_status,
                        "environment": environment,
                    },
                    account_list_params.AccountListParams,
                ),
            ),
            model=Account,
        )


class AccountsResourceWithRawResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = to_raw_response_wrapper(
            accounts.retrieve,
        )
        self.list = to_raw_response_wrapper(
            accounts.list,
        )


class AsyncAccountsResourceWithRawResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = async_to_raw_response_wrapper(
            accounts.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            accounts.list,
        )


class AccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = to_streamed_response_wrapper(
            accounts.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            accounts.list,
        )


class AsyncAccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = async_to_streamed_response_wrapper(
            accounts.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            accounts.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["RuleListParams"]


class RuleListParams(TypedDict, total=False):
    account_slug: str
    """Account slug"""

    cursor: str
    """The pagination cursor value."""

    customer_slug: str
    """Customer slug"""

    enabled: bool

    is_in_split_test: bool

    section_id: int
    """Section ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .steps import (
    StepsResource,
    AsyncStepsResource,
    StepsResourceWithRawResponse,
    AsyncStepsResourceWithRawResponse,
    StepsResourceWithStreamingResponse,
    AsyncStepsResourceWithStreamingResponse,
)
from .values import (
    ValuesResource,
    AsyncValuesResource,
    ValuesResourceWithRawResponse,
    AsyncValuesResourceWithRawResponse,
    ValuesResourceWithStreamingResponse,
    AsyncValuesResourceWithStreamingResponse,
)
from .accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)
from .sections import (
    SectionsResource,
    AsyncSectionsResource,
    SectionsResourceWithRawResponse,
    AsyncSectionsResourceWithRawResponse,
    SectionsResourceWithStreamingResponse,
    AsyncSectionsResourceWithStreamingResponse,
)
from .customers import (
    CustomersResource,
    AsyncCustomersResource,
    CustomersResourceWithRawResponse,
    AsyncCustomersResourceWithRawResponse,
    CustomersResourceWithStreamingResponse,
    AsyncCustomersResourceWithStreamingResponse,
)
from .experiments import (
    ExperimentsResource,
    AsyncExperimentsResource,
    ExperimentsResourceWithRawResponse,
    AsyncExperimentsResourceWithRawResponse,
    ExperimentsResourceWithStreamingResponse,
    AsyncExperimentsResourceWithStreamingResponse,
)
from .seo_experiment_results import (
    SeoExperimentResultsResource,
    AsyncSeoExperimentResultsResource,
    SeoExperimentResultsResourceWithRawResponse,
    AsyncSeoExperimentResultsResourceWithRawResponse,
    SeoExperimentResultsResourceWithStreamingResponse,
    AsyncSeoExperimentResultsResourceWithStreamingResponse,
)

__all__ = [
    "CustomersResource",
    "AsyncCustomersResource",
    "CustomersResourceWithRawResponse",
    "AsyncCustomersResourceWithRawResponse",
    "CustomersResourceWithStreamingResponse",
    "AsyncCustomersResourceWithStreamingResponse",
    "AccountsResource",
    "AsyncAccountsResource",
    "AccountsResourceWithRawResponse",
    "AsyncAccountsResourceWithRawResponse",
    "AccountsResourceWithStreamingResponse",
    "AsyncAccountsResourceWithStreamingResponse",
    "SectionsResource",
    "AsyncSectionsResource",
    "SectionsResourceWithRawResponse",
    "AsyncSectionsResourceWithRawResponse",
    "SectionsResourceWithStreamingResponse",
    "AsyncSectionsResourceWithStreamingResponse",
    "RulesResource",
    "AsyncRulesResource",
    "RulesResourceWithRawResponse",
    "AsyncRulesResourceWithRawResponse",
    "RulesResourceWithStreamingResponse",
    "AsyncRulesResourceWithStreamingResponse",
    "StepsResource",
    "AsyncStepsResource",
    "StepsResourceWithRawResponse",
    "AsyncStepsResourceWithRawResponse",
    "StepsResourceWithStreamingResponse",
    "AsyncStepsResourceWithStreamingResponse",
    "ValuesResource",
    "AsyncValuesResource",
    "ValuesResourceWithRawResponse",
    "AsyncValuesResourceWithRawResponse",
    "ValuesResourceWithStreamingResponse",
    "AsyncValuesResourceWithStreamingResponse",
    "ExperimentsResource",
    "AsyncExperimentsResource",
    "ExperimentsResourceWithRawResponse",
    "AsyncExperimentsResourceWithRawResponse",
    "ExperimentsResourceWithStreamingResponse",
    "AsyncExperimentsResourceWithStreamingResponse",
    "SeoExperimentResultsResource",
    "AsyncSeoExperimentResultsResource",
    "SeoExperimentResultsResourceWithRawResponse",
    "AsyncSeoExperimentResultsResourceWithRawResponse",
    "SeoExperimentResultsResourceWithStreamingResponse",
    "AsyncSeoExperimentResultsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["Rule"]


class Rule(BaseModel):
    id: int

    account_slug: str

    customer_slug: str

    enabled: bool

    is_in_split_test: bool

    name: str

    section_id: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import step_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorURLPage, AsyncCursorURLPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.step import Step

__all__ = ["StepsResource", "AsyncStepsResource"]


class StepsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> StepsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#accessing-raw-response-data-eg-headers
        """
        return StepsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StepsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#with_streaming_response
        """
        return StepsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        step_id: int,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Step:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            f"/api/external/v1/steps/{step_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Step,
        )

    def list(
        self,
        *,
        account_slug: str | Omit = omit,
        cursor: str | Omit = omit,
        customer_slug: str | Omit = omit,
        enabled: bool | Omit = omit,
        rule_id: int | Omit = omit,
        section_id: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[Step]:
        """
        Args:
          account_slug: Account slug

          cursor: The pagination cursor value.

          customer_slug: Customer slug

          rule_id: Rule ID

          section_id: Section ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/external/v1/steps/",
            page=SyncCursorURLPage[Step],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_slug": account_slug,
                        "cursor": cursor,
                        "customer_slug": customer_slug,
                        "enabled": enabled,
                        "rule_id": rule_id,
                        "section_id": section_id,
                    },
                    step_list_params.StepListParams,
                ),
            ),
            model=Step,
        )


class AsyncStepsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncStepsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStepsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStepsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#with_streaming_response
        """
        return AsyncStepsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        step_id: int,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Step:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            f"/api/external/v1/steps/{step_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Step,
        )

    def list(
        self,
        *,
        account_slug: str | Omit = omit,
        cursor: str | Omit = omit,
        customer_slug: str | Omit = omit,
        enabled: bool | Omit = omit,
        rule_id: int | Omit = omit,
        section_id: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Step, AsyncCursorURLPage[Step]]:
        """
        Args:
          account_slug: Account slug

          cursor: The pagination cursor value.

          customer_slug: Customer slug

          rule_id: Rule ID

          section_id: Section ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/external/v1/steps/",
            page=AsyncCursorURLPage[Step],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_slug": account_slug,
                        "cursor": cursor,
                        "customer_slug": customer_slug,
                        "enabled": enabled,
                        "rule_id": rule_id,
                        "section_id": section_id,
                    },
                    step_list_params.StepListParams,
                ),
            ),
            model=Step,
        )


class StepsResourceWithRawResponse:
    def __init__(self, steps: StepsResource) -> None:
        self._steps = steps

        self.retrieve = to_raw_response_wrapper(
            steps.retrieve,
        )
        self.list = to_raw_response_wrapper(
            steps.list,
        )


class AsyncStepsResourceWithRawResponse:
    def __init__(self, steps: AsyncStepsResource) -> None:
        self._steps = steps

        self.retrieve = async_to_raw_response_wrapper(
            steps.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            steps.list,
        )


class StepsResourceWithStreamingResponse:
    def __init__(self, steps: StepsResource) -> None:
        self._steps = steps

        self.retrieve = to_streamed_response_wrapper(
            steps.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            steps.list,
        )


class AsyncStepsResourceWithStreamingResponse:
    def __init__(self, steps: AsyncStepsResource) -> None:
        self._steps = steps

        self.retrieve = async_to_streamed_response_wrapper(
            steps.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            steps.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import customer_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorURLPage, AsyncCursorURLPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.customer import Customer

__all__ = ["CustomersResource", "AsyncCustomersResource"]


class CustomersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#accessing-raw-response-data-eg-headers
        """
        return CustomersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#with_streaming_response
        """
        return CustomersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        customer_slug: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Customer:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_slug:
            raise ValueError(f"Expected a non-empty value for `customer_slug` but received {customer_slug!r}")
        return self._get(
            f"/api/external/v1/customers/{customer_slug}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Customer,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[Customer]:
        """
        Args:
          cursor: The pagination cursor value.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/external/v1/customers/",
            page=SyncCursorURLPage[Customer],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"cursor": cursor}, customer_list_params.CustomerListParams),
            ),
            model=Customer,
        )


class AsyncCustomersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/adamsteele-sp/searchpilot-python#with_streaming_response
        """
        return AsyncCustomersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        customer_slug: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Customer:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_slug:
            raise ValueError(f"Expected a non-empty value for `customer_slug` but received {customer_slug!r}")
        return await self._get(
            f"/api/external/v1/customers/{customer_slug}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Customer,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Customer, AsyncCursorURLPage[Customer]]:
        """
        Args:
          cursor: The pagination cursor value.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/external/v1/customers/",
            page=AsyncCursorURLPage[Customer],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"cursor": cursor}, customer_list_params.CustomerListParams),
            ),
            model=Customer,
        )


class CustomersResourceWithRawResponse:
    def __init__(self, customers: CustomersResource) -> None:
        self._customers = customers

        self.retrieve = to_raw_response_wrapper(
            customers.retrieve,
        )
        self.list = to_raw_response_wrapper(
            customers.list,
        )


class AsyncCustomersResourceWithRawResponse:
    def __init__(self, customers: AsyncCustomersResource) -> None:
        self._customers = customers

        self.retrieve = async_to_raw_response_wrapper(
            customers.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            customers.list,
        )


class CustomersResourceWithStreamingResponse:
    def __init__(self, customers: CustomersResource) -> None:
        self._customers = customers

        self.retrieve = to_streamed_response_wrapper(
            customers.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            customers.list,
        )


class AsyncCustomersResourceWithStreamingResponse:
    def __init__(self, customers: AsyncCustomersResource) -> None:
        self._customers = customers

        self.retrieve = async_to_streamed_response_wrapper(
            customers.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            customers.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rule import Rule as Rule
from .step import Step as Step
from .value import Value as Value
from .account import Account as Account
from .section import Section as Section
from .customer import Customer as Customer
from .experiment import Experiment as Experiment
from .seo_experiment_result import SeoExperimentResult as SeoExperimentResult



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Step"]


class Step(BaseModel):
    id: int

    account_slug: str

    adaptation: Literal[
        "carbon.adjusters.body.adjusters.add_before_css_selector",
        "carbon.adjusters.body.adjusters.prepend_to_css_selector",
        "carbon.adjusters.body.adjusters.append_to_css_selector",
        "carbon.adjusters.body.adjusters.add_after_css_selector",
        "carbon.adjusters.body.adjusters.add_class",
        "carbon.adjusters.body.adjusters.set_attribute_value",
        "carbon.adjusters.body.adjusters.update_element_type",
        "carbon.adjusters.body.adjusters.replace_inner_css_selector",
        "carbon.adjusters.body.adjusters.replace_inner_css_selector_with_text",
        "carbon.adjusters.body.adjusters.find_replace_within",
        "carbon.adjusters.body.adjusters.remove_css_selector",
        "carbon.adjusters.body.adjusters.remove_class",
        "carbon.adjusters.body.adjusters.remove_attribute",
        "carbon.adjusters.body.adjusters.set_json_element",
        "carbon.adjusters.body.adjusters.update_json_element",
        "carbon.adjusters.body.adjusters.delete_json_element",
        "carbon.adjusters.body.adjusters.insert_element_existing_json_array",
        "carbon.adjusters.headers.adjusters.header_append",
        "carbon.adjusters.headers.adjusters.set_header",
        "carbon.adjusters.headers.adjusters.update_header",
        "carbon.adjusters.headers.adjusters.remove_header",
        "carbon.adjusters.headers.adjusters.remove_header_value_by_regex",
        "carbon.adjusters.headers.adjusters.change_status_code",
        "carbon.adjusters.headers.adjusters.path_redirect",
    ]
    """
    - `carbon.adjusters.body.adjusters.add_before_css_selector` - Insert block of
      HTML - before the element
    - `carbon.adjusters.body.adjusters.prepend_to_css_selector` - Insert block of
      HTML - after the opening tag
    - `carbon.adjusters.body.adjusters.append_to_css_selector` - Insert block of
      HTML - before the closing tag
    - `carbon.adjusters.body.adjusters.add_after_css_selector` - Insert block of
      HTML - after the element
    - `carbon.adjusters.body.adjusters.add_class` - Add additional class to an
      element
    - `carbon.adjusters.body.adjusters.set_attribute_value` - Set or update an
      attribute of an element
    - `carbon.adjusters.body.adjusters.update_element_type` - Update HTML tag type
      (e.g. from h1 to h2)
    - `carbon.adjusters.body.adjusters.replace_inner_css_selector` - Replace
      contents of an element with HTML
    - `carbon.adjusters.body.adjusters.replace_inner_css_selector_with_text` -
      Replace contents of an element with text
    - `carbon.adjusters.body.adjusters.find_replace_within` - Find & replace over an
      element
    - `carbon.adjusters.body.adjusters.remove_css_selector` - Remove an element
    - `carbon.adjusters.body.adjusters.remove_class` - Remove a class from an
      element
    - `carbon.adjusters.body.adjusters.remove_attribute` - Remove an attribute of an
      element
    - `carbon.adjusters.body.adjusters.set_json_element` - Set the value of a JSON
      element
    - `carbon.adjusters.body.adjusters.update_json_element` - Update the value of a
      JSON element
    - `carbon.adjusters.body.adjusters.delete_json_element` - Delete an element from
      a JSON block
    - `carbon.adjusters.body.adjusters.insert_element_existing_json_array` - Insert
      element into existing JSON array
    - `carbon.adjusters.headers.adjusters.header_append` - Append an additional
      value to a header
    - `carbon.adjusters.headers.adjusters.set_header` - Set the value of an HTTP
      Header
    - `carbon.adjusters.headers.adjusters.update_header` - Overwrite the value of an
      existing header
    - `carbon.adjusters.headers.adjusters.remove_header` - Remove a Header
    - `carbon.adjusters.headers.adjusters.remove_header_value_by_regex` - Find and
      replace on Header contents
    - `carbon.adjusters.headers.adjusters.change_status_code` - Change the status
      code of the response
    - `carbon.adjusters.headers.adjusters.path_redirect` - Redirect to a new URL
    """

    customer_slug: str

    enabled: bool

    name: str
    """Descriptive name to help identify the Step"""

    parameters: object

    rule_id: int

    section_id: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import datetime
from typing import Dict, List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SeoExperimentResult", "Metric", "Pipeline"]


class Metric(BaseModel):
    id: int

    name: str


class Pipeline(BaseModel):
    id: int

    is_preferred: bool

    is_published: bool

    name: str

    steps: object


class SeoExperimentResult(BaseModel):
    id: int

    account_slug: str

    alpha: float

    customer_slug: str

    date: datetime.date

    dimension_combination: Dict[str, str]

    estimated_wait_time: Optional[Literal["no_valid_wait_time", "up_to_a_week", "up_to_two_weeks"]] = None

    expected_overall_extra_metric_value: Optional[float] = None

    expected_overall_extra_metric_value_lower: Optional[float] = None

    expected_overall_extra_metric_value_upper: Optional[float] = None

    experiment_id: int

    extra_alphas: List[float]

    intervention_date: datetime.date

    is_significant: bool

    metric: Metric

    pipeline: Pipeline

    post_pipeline_average_daily_metric_value: Optional[float] = None

    recommendation: str

    section_id: Optional[int] = None

    steps: object

    bucket_correlation: Optional[float] = None

    coefficient_of_determination: Optional[float] = None

    confidence: Optional[float] = None

    days_running: Optional[int] = None

    end_date: Optional[datetime.date] = None

    expected_uplift_percent: Optional[float] = None

    expected_uplift_percent_lower: Optional[float] = None

    expected_uplift_percent_upper: Optional[float] = None

    forecast_reliability_score: Optional[float] = None

    generated_on: Optional[datetime.datetime] = None

    lower_percentage: Optional[float] = None

    mean_absolute_percentage_error: Optional[float] = None

    metric_value: Optional[float] = None

    monthly_change: Optional[float] = None

    monthly_lower: Optional[float] = None

    monthly_upper: Optional[float] = None

    percentage_change: Optional[float] = None

    post_pipeline_number_unique_paths: Optional[int] = None

    post_pipeline_ready_to_test_score: Optional[float] = None

    self_uplift: Optional[float] = None

    start_date: Optional[datetime.date] = None

    upper_percentage: Optional[float] = None

    volatility_index: Optional[float] = None

    yearly_change: Optional[float] = None

    yearly_lower: Optional[float] = None

    yearly_upper: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Generic, TypeVar, Optional
from typing_extensions import override

import httpx

from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = ["SyncCursorURLPage", "AsyncCursorURLPage"]

_T = TypeVar("_T")


class SyncCursorURLPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    results: List[_T]
    next: Optional[str] = None

    @override
    def _get_page_items(self) -> List[_T]:
        results = self.results
        if not results:
            return []
        return results

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        url = self.next
        if url is None:
            return None

        return PageInfo(url=httpx.URL(url))


class AsyncCursorURLPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    results: List[_T]
    next: Optional[str] = None

    @override
    def _get_page_items(self) -> List[_T]:
        results = self.results
        if not results:
            return []
        return results

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        url = self.next
        if url is None:
            return None

        return PageInfo(url=httpx.URL(url))



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["ExperimentListParams"]


class ExperimentListParams(TypedDict, total=False):
    account_slug: str
    """Account slug"""

    complete_state: Optional[Literal["accepted", "hidden", "invalid", "null", "rejected"]]
    """
    - `accepted` - Positive
    - `rejected` - Negative
    - `null` - Inconclusive
    - `invalid` - Invalid
    - `hidden` - Hidden
    """

    cursor: str
    """The pagination cursor value."""

    customer_slug: str
    """Customer slug"""

    ended_at_after: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Ended at"""

    ended_at_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Ended at"""

    has_ended: bool
    """Has Ended"""

    has_started: bool
    """Has Started"""

    section_id: int
    """Section ID"""

    started_at_after: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Started At"""

    started_at_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Started At"""

    status: Literal[
        "ended",
        "failed-salt-shake",
        "paused",
        "published",
        "ready",
        "rolledout",
        "waiting-for-end",
        "waiting-for-manual-salt",
        "waiting-for-rollout",
        "waiting-for-salt",
    ]
    """
    - `waiting-for-salt` - Waiting for salt
    - `waiting-for-manual-salt` - Waiting for manual salt
    - `failed-salt-shake` - Failed salt shake
    - `ready` - Ready
    - `published` - Published
    - `paused` - Paused
    - `waiting-for-rollout` - Waiting for rollout
    - `rolledout` - Rolled out to 100%
    - `waiting-for-end` - Waiting for end
    - `ended` - Ended
    """

    tags_all: SequenceNotStr[str]
    """Filter for experiments who match ALL tag names"""

    tags_any: SequenceNotStr[str]
    """Filter for experiments who match ANY tag names"""

    test_type: Literal["cro", "full_funnel", "linear_full_funnel", "seo"]
    """
    - `seo` - SEO
    - `cro` - CRO
    - `full_funnel` - Full Funnel
    - `linear_full_funnel` - Linear Full Funnel
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SeoExperimentResultListParams"]


class SeoExperimentResultListParams(TypedDict, total=False):
    experiment_id: Required[int]
    """Experiment ID"""

    cursor: str
    """The pagination cursor value."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Section", "CurrentExperiment", "LatestReadyToTestResult"]


class CurrentExperiment(BaseModel):
    id: int


class LatestReadyToTestResult(BaseModel):
    average_daily_metric_value_30: float

    coefficient_of_variance_30: float

    max_bucket_metric_value_range_mean: float

    num_days_missing_data: int

    number_unique_pages_30: float

    requested_at: datetime

    score: float

    score_breakdown_control_avg_daily_metric_value: float

    score_breakdown_correlation_mean: float

    score_breakdown_correlation_stddev: float

    score_breakdown_cov_diff_daily_metric_value: float

    score_breakdown_sufficient_path_score: float

    score_breakdown_variant_avg_daily_metric_value: float


class Section(BaseModel):
    id: int

    account_slug: str

    current_experiment: Optional[CurrentExperiment] = None

    customer_slug: str

    latest_ready_to_test_result: Optional[LatestReadyToTestResult] = None

    matching_expression: Dict[str, object]

    name: str

    params_to_keep: List[str]

    slug: str

    published_to_live: Optional[bool] = None

    type: Optional[Literal["response", "api", "request"]] = None
    """
    - `response` - Response
    - `api` - API
    - `request` - Request
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Value", "Conversion"]


class Conversion(BaseModel):
    step: Literal[
        "carbon.converters.converters.to_lower",
        "carbon.converters.converters.to_upper",
        "carbon.converters.converters.to_title_case",
        "carbon.converters.converters.strip",
        "carbon.converters.converters.extract_regex",
        "carbon.converters.converters.regex_replace",
        "carbon.converters.converters.split_to_array",
        "carbon.converters.converters.slice",
        "carbon.converters.converters.extract_from_json",
        "carbon.converters.converters.first_element",
        "carbon.converters.converters.last_element",
        "carbon.converters.converters.largest",
        "carbon.converters.converters.smallest",
    ]
    """
    - `carbon.converters.converters.to_lower` - Convert to lower case
    - `carbon.converters.converters.to_upper` - Convert to upper case
    - `carbon.converters.converters.to_title_case` - Convert to title case
    - `carbon.converters.converters.strip` - Strip whitespace
    - `carbon.converters.converters.extract_regex` - Find Value using regex
    - `carbon.converters.converters.regex_replace` - Replace value using regex
    - `carbon.converters.converters.split_to_array` - Split the Value into an array
      of values
    - `carbon.converters.converters.slice` - Slice (only keep part) of the Value
    - `carbon.converters.converters.extract_from_json` - Extract from block of JSON
    - `carbon.converters.converters.first_element` - First element in the array
    - `carbon.converters.converters.last_element` - Last element in the array
    - `carbon.converters.converters.largest` - The largest element in the array
    - `carbon.converters.converters.smallest` - The smallest element in the array
    """

    step_parameters: object


class Value(BaseModel):
    id: int

    account_slug: str

    conversions: List[Conversion]

    customer_slug: str

    name: str
    """Can only contain lowercase alphanumerics and underscores."""

    section_id: int

    source: Literal[
        "carbon.extractors.extractors.css_selector_attribute",
        "carbon.extractors.extractors.match_response_header_key",
        "carbon.extractors.extractors.match_request_header_key",
        "carbon.extractors.extractors.match_hostname",
        "carbon.extractors.extractors.match_url",
        "carbon.extractors.extractors.count",
        "carbon.extractors.extractors.from_dataset",
        "carbon.extractors.extractors.constant",
        "carbon.extractors.extractors.generate_random",
        "request_constant",
        "request_uri",
        "request_hostname",
        "request_get_parameter",
        "request_get_header",
        "from_dataset",
    ]
    """
    - `carbon.extractors.extractors.css_selector_attribute` - Extract the value from
      the HTML of the page being requested
    - `carbon.extractors.extractors.match_response_header_key` - Extract the value
      from the HTTP response Headers of the page being requested
    - `carbon.extractors.extractors.match_request_header_key` - Extract the value
      from the HTTP request Headers of the page being requested
    - `carbon.extractors.extractors.match_hostname` - Extract the hostname from the
      URL of the page being requested
    - `carbon.extractors.extractors.match_url` - Extract the value from the URL of
      the page being requested
    - `carbon.extractors.extractors.count` - Extract a count of elements from the
      HTML
    - `carbon.extractors.extractors.from_dataset` - Extract a value from a dataset
    - `carbon.extractors.extractors.constant` - I'll enter the value manually
    - `carbon.extractors.extractors.generate_random` - Generate a random number
      based on the canonical url of the page
    - `request_constant` - Constant
    - `request_uri` - Extract from request path
    - `request_hostname` - Extract from request hostname
    - `request_get_parameter` - Extract from a request parameter
    - `request_get_header` - Extract from a request header
    - `from_dataset` - Extract a value from a dataset
    """

    source_parameters: object

    description: Optional[str] = None
    """Descriptive name that identifies the Value"""

    is_in_preview_spec: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["SectionListParams"]


class SectionListParams(TypedDict, total=False):
    account_slug: str
    """Account slug"""

    cursor: str
    """The pagination cursor value."""

    customer_slug: str
    """Customer slug"""

    published_to_live: bool

    type: Literal["api", "request", "response"]
    """
    - `response` - Response
    - `api` - API
    - `request` - Request
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = [
    "Experiment",
    "CroDetails",
    "SeoDetails",
    "SeoDetailsExperimentMetric",
    "SeoDetailsExperimentMetricMetric",
    "SeoDetailsLatestPrimarySeoExperimentResult",
    "SeoDetailsPreferredPipeline",
    "Tag",
]


class CroDetails(BaseModel):
    cro_parameters: Dict[str, object]

    hasher_parameters: Dict[str, object]

    hasher: Optional[
        Literal[
            "carbon.hashers.base.simple",
            "carbon.hashers.base.random_by_percent",
            "carbon.hashers.base.random_by_percent_2",
            "carbon.hashers.base.random_by_percent_3",
            "carbon.hashers.base.random_by_percent_4",
            "carbon.hashers.base.random_by_percent_5",
            "carbon.hashers.base.random_by_normalised",
            "carbon.hashers.base.hash_variable",
            "carbon.hashers.base.variable_matches",
            "carbon.hashers.base.cro_random_user",
            "carbon.hashers.base.cro_always_control",
        ]
    ] = None
    """
    - `carbon.hashers.base.simple` - Simple
    - `carbon.hashers.base.random_by_percent` - Random (deprecated)
    - `carbon.hashers.base.random_by_percent_2` - Random (deprecated)
    - `carbon.hashers.base.random_by_percent_3` - Split by URL path into
      statistically similar buckets (deprecated)
    - `carbon.hashers.base.random_by_percent_4` - Split by normalised URL path into
      statistically similar buckets (deprecated)
    - `carbon.hashers.base.random_by_percent_5` - Split by normalised URL path into
      statistically similar buckets (recommended)
    - `carbon.hashers.base.random_by_normalised` - Split by normalised URL [path or
      canonical] into statistically similar buckets
    - `carbon.hashers.base.hash_variable` - Split by Value into random buckets
    - `carbon.hashers.base.variable_matches` - Split by Value into manual buckets
    - `carbon.hashers.base.cro_random_user` - CRO Test
    - `carbon.hashers.base.cro_always_control` - CRO Test - always serve control
    """


class SeoDetailsExperimentMetricMetric(BaseModel):
    id: int

    name: str


class SeoDetailsExperimentMetric(BaseModel):
    is_primary: bool

    metric: SeoDetailsExperimentMetricMetric


class SeoDetailsLatestPrimarySeoExperimentResult(BaseModel):
    id: int


class SeoDetailsPreferredPipeline(BaseModel):
    id: int

    name: str

    steps: object


class SeoDetails(BaseModel):
    bucket_urls_csv_url: Optional[str] = None

    experiment_metrics: List[SeoDetailsExperimentMetric]

    hasher_parameters: Dict[str, object]

    latest_primary_seo_experiment_result: Optional[SeoDetailsLatestPrimarySeoExperimentResult] = None

    preferred_pipeline: SeoDetailsPreferredPipeline

    hasher: Optional[
        Literal[
            "carbon.hashers.base.simple",
            "carbon.hashers.base.random_by_percent",
            "carbon.hashers.base.random_by_percent_2",
            "carbon.hashers.base.random_by_percent_3",
            "carbon.hashers.base.random_by_percent_4",
            "carbon.hashers.base.random_by_percent_5",
            "carbon.hashers.base.random_by_normalised",
            "carbon.hashers.base.hash_variable",
            "carbon.hashers.base.variable_matches",
            "carbon.hashers.base.cro_random_user",
            "carbon.hashers.base.cro_always_control",
        ]
    ] = None
    """
    - `carbon.hashers.base.simple` - Simple
    - `carbon.hashers.base.random_by_percent` - Random (deprecated)
    - `carbon.hashers.base.random_by_percent_2` - Random (deprecated)
    - `carbon.hashers.base.random_by_percent_3` - Split by URL path into
      statistically similar buckets (deprecated)
    - `carbon.hashers.base.random_by_percent_4` - Split by normalised URL path into
      statistically similar buckets (deprecated)
    - `carbon.hashers.base.random_by_percent_5` - Split by normalised URL path into
      statistically similar buckets (recommended)
    - `carbon.hashers.base.random_by_normalised` - Split by normalised URL [path or
      canonical] into statistically similar buckets
    - `carbon.hashers.base.hash_variable` - Split by Value into random buckets
    - `carbon.hashers.base.variable_matches` - Split by Value into manual buckets
    - `carbon.hashers.base.cro_random_user` - CRO Test
    - `carbon.hashers.base.cro_always_control` - CRO Test - always serve control
    """

    primary_alpha: Optional[float] = None

    salt_shake_requested_at: Optional[datetime] = None


class Tag(BaseModel):
    name: str


class Experiment(BaseModel):
    id: int

    account_slug: str

    complete_state: Optional[Literal["accepted", "rejected", "null", "invalid", "hidden"]] = None

    cro_details: Optional[CroDetails] = None

    customer_slug: str

    ended_at: Optional[datetime] = None

    name: str
    """The name of the test"""

    section_id: Optional[int] = None

    section_type: Literal["request", "response"]
    """
    - `request` - Request
    - `response` - Response
    """

    seo_details: Optional[SeoDetails] = None

    started_at: Optional[datetime] = None

    tags: List[Tag]

    test_type: Literal["seo", "cro", "full_funnel", "linear_full_funnel"]
    """
    - `seo` - SEO
    - `cro` - CRO
    - `full_funnel` - Full Funnel
    - `linear_full_funnel` - Linear Full Funnel
    """

    hypothesis: Optional[str] = None

    notes: Optional[str] = None
    """Add any notes about the test"""

    published_section_name: Optional[str] = None

    status: Optional[
        Literal[
            "waiting-for-salt",
            "waiting-for-manual-salt",
            "failed-salt-shake",
            "ready",
            "published",
            "paused",
            "waiting-for-rollout",
            "rolledout",
            "waiting-for-end",
            "ended",
        ]
    ] = None
    """
    - `waiting-for-salt` - Waiting for salt
    - `waiting-for-manual-salt` - Waiting for manual salt
    - `failed-salt-shake` - Failed salt shake
    - `ready` - Ready
    - `published` - Published
    - `paused` - Paused
    - `waiting-for-rollout` - Waiting for rollout
    - `rolledout` - Rolled out to 100%
    - `waiting-for-end` - Waiting for end
    - `ended` - Ended
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .shared import (
    Rule as Rule,
    Step as Step,
    Value as Value,
    Account as Account,
    Section as Section,
    Customer as Customer,
    Experiment as Experiment,
    SeoExperimentResult as SeoExperimentResult,
)
from .rule_list_params import RuleListParams as RuleListParams
from .step_list_params import StepListParams as StepListParams
from .value_list_params import ValueListParams as ValueListParams
from .account_list_params import AccountListParams as AccountListParams
from .section_list_params import SectionListParams as SectionListParams
from .customer_list_params import CustomerListParams as CustomerListParams
from .experiment_list_params import ExperimentListParams as ExperimentListParams
from .seo_experiment_result_list_params import SeoExperimentResultListParams as SeoExperimentResultListParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import rules, steps, values, accounts, sections, customers, experiments, seo_experiment_results
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, SearchPilotError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "SearchPilot",
    "AsyncSearchPilot",
    "Client",
    "AsyncClient",
]


class SearchPilot(SyncAPIClient):
    customers: customers.CustomersResource
    accounts: accounts.AccountsResource
    sections: sections.SectionsResource
    rules: rules.RulesResource
    steps: steps.StepsResource
    values: values.ValuesResource
    experiments: experiments.ExperimentsResource
    seo_experiment_results: seo_experiment_results.SeoExperimentResultsResource
    with_raw_response: SearchPilotWithRawResponse
    with_streaming_response: SearchPilotWithStreamedResponse

    # client options
    api_jwt: str

    def __init__(
        self,
        *,
        api_jwt: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous SearchPilot client instance.

        This automatically infers the `api_jwt` argument from the `SEARCHPILOT_API_JWT` environment variable if it is not provided.
        """
        if api_jwt is None:
            api_jwt = os.environ.get("SEARCHPILOT_API_JWT")
        if api_jwt is None:
            raise SearchPilotError(
                "The api_jwt client option must be set either by passing api_jwt to the client or by setting the SEARCHPILOT_API_JWT environment variable"
            )
        self.api_jwt = api_jwt

        if base_url is None:
            base_url = os.environ.get("SEARCHPILOT_BASE_URL")
        if base_url is None:
            base_url = f"https://app.searchpilot.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.customers = customers.CustomersResource(self)
        self.accounts = accounts.AccountsResource(self)
        self.sections = sections.SectionsResource(self)
        self.rules = rules.RulesResource(self)
        self.steps = steps.StepsResource(self)
        self.values = values.ValuesResource(self)
        self.experiments = experiments.ExperimentsResource(self)
        self.seo_experiment_results = seo_experiment_results.SeoExperimentResultsResource(self)
        self.with_raw_response = SearchPilotWithRawResponse(self)
        self.with_streaming_response = SearchPilotWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="repeat")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_jwt = self.api_jwt
        return {"Authorization": f"Bearer {api_jwt}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_jwt: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_jwt=api_jwt or self.api_jwt,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncSearchPilot(AsyncAPIClient):
    customers: customers.AsyncCustomersResource
    accounts: accounts.AsyncAccountsResource
    sections: sections.AsyncSectionsResource
    rules: rules.AsyncRulesResource
    steps: steps.AsyncStepsResource
    values: values.AsyncValuesResource
    experiments: experiments.AsyncExperimentsResource
    seo_experiment_results: seo_experiment_results.AsyncSeoExperimentResultsResource
    with_raw_response: AsyncSearchPilotWithRawResponse
    with_streaming_response: AsyncSearchPilotWithStreamedResponse

    # client options
    api_jwt: str

    def __init__(
        self,
        *,
        api_jwt: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncSearchPilot client instance.

        This automatically infers the `api_jwt` argument from the `SEARCHPILOT_API_JWT` environment variable if it is not provided.
        """
        if api_jwt is None:
            api_jwt = os.environ.get("SEARCHPILOT_API_JWT")
        if api_jwt is None:
            raise SearchPilotError(
                "The api_jwt client option must be set either by passing api_jwt to the client or by setting the SEARCHPILOT_API_JWT environment variable"
            )
        self.api_jwt = api_jwt

        if base_url is None:
            base_url = os.environ.get("SEARCHPILOT_BASE_URL")
        if base_url is None:
            base_url = f"https://app.searchpilot.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.customers = customers.AsyncCustomersResource(self)
        self.accounts = accounts.AsyncAccountsResource(self)
        self.sections = sections.AsyncSectionsResource(self)
        self.rules = rules.AsyncRulesResource(self)
        self.steps = steps.AsyncStepsResource(self)
        self.values = values.AsyncValuesResource(self)
        self.experiments = experiments.AsyncExperimentsResource(self)
        self.seo_experiment_results = seo_experiment_results.AsyncSeoExperimentResultsResource(self)
        self.with_raw_response = AsyncSearchPilotWithRawResponse(self)
        self.with_streaming_response = AsyncSearchPilotWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="repeat")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_jwt = self.api_jwt
        return {"Authorization": f"Bearer {api_jwt}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_jwt: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_jwt=api_jwt or self.api_jwt,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class SearchPilotWithRawResponse:
    def __init__(self, client: SearchPilot) -> None:
        self.customers = customers.CustomersResourceWithRawResponse(client.customers)
        self.accounts = accounts.AccountsResourceWithRawResponse(client.accounts)
        self.sections = sections.SectionsResourceWithRawResponse(client.sections)
        self.rules = rules.RulesResourceWithRawResponse(client.rules)
        self.steps = steps.StepsResourceWithRawResponse(client.steps)
        self.values = values.ValuesResourceWithRawResponse(client.values)
        self.experiments = experiments.ExperimentsResourceWithRawResponse(client.experiments)
        self.seo_experiment_results = seo_experiment_results.SeoExperimentResultsResourceWithRawResponse(
            client.seo_experiment_results
        )


class AsyncSearchPilotWithRawResponse:
    def __init__(self, client: AsyncSearchPilot) -> None:
        self.customers = customers.AsyncCustomersResourceWithRawResponse(client.customers)
        self.accounts = accounts.AsyncAccountsResourceWithRawResponse(client.accounts)
        self.sections = sections.AsyncSectionsResourceWithRawResponse(client.sections)
        self.rules = rules.AsyncRulesResourceWithRawResponse(client.rules)
        self.steps = steps.AsyncStepsResourceWithRawResponse(client.steps)
        self.values = values.AsyncValuesResourceWithRawResponse(client.values)
        self.experiments = experiments.AsyncExperimentsResourceWithRawResponse(client.experiments)
        self.seo_experiment_results = seo_experiment_results.AsyncSeoExperimentResultsResourceWithRawResponse(
            client.seo_experiment_results
        )


class SearchPilotWithStreamedResponse:
    def __init__(self, client: SearchPilot) -> None:
        self.customers = customers.CustomersResourceWithStreamingResponse(client.customers)
        self.accounts = accounts.AccountsResourceWithStreamingResponse(client.accounts)
        self.sections = sections.SectionsResourceWithStreamingResponse(client.sections)
        self.rules = rules.RulesResourceWithStreamingResponse(client.rules)
        self.steps = steps.StepsResourceWithStreamingResponse(client.steps)
        self.values = values.ValuesResourceWithStreamingResponse(client.values)
        self.experiments = experiments.ExperimentsResourceWithStreamingResponse(client.experiments)
        self.seo_experiment_results = seo_experiment_results.SeoExperimentResultsResourceWithStreamingResponse(
            client.seo_experiment_results
        )


class AsyncSearchPilotWithStreamedResponse:
    def __init__(self, client: AsyncSearchPilot) -> None:
        self.customers = customers.AsyncCustomersResourceWithStreamingResponse(client.customers)
        self.accounts = accounts.AsyncAccountsResourceWithStreamingResponse(client.accounts)
        self.sections = sections.AsyncSectionsResourceWithStreamingResponse(client.sections)
        self.rules = rules.AsyncRulesResourceWithStreamingResponse(client.rules)
        self.steps = steps.AsyncStepsResourceWithStreamingResponse(client.steps)
        self.values = values.AsyncValuesResourceWithStreamingResponse(client.values)
        self.experiments = experiments.AsyncExperimentsResourceWithStreamingResponse(client.experiments)
        self.seo_experiment_results = seo_experiment_results.AsyncSeoExperimentResultsResourceWithStreamingResponse(
            client.seo_experiment_results
        )


Client = SearchPilot

AsyncClient = AsyncSearchPilot
